/*
 * A JavaScript implementation of the RSA Data Security, Inc. MD5 Message
 * Digest Algorithm, as defined in RFC 1321.
 * Version 2.1 Copyright (C) Paul Johnston 1999 - 2002.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for more info.
 *
 * Class wrapper by Matt Newberry - 2004-03-13
 */
 
// Helper Functions
// loginform must have fields named "password", "wildcard", and "hash"; password field will be blanked out,
// LoginMD5 hash field will contain hash of wildcard and password; LoginSHA1 hash field will contain hash 
// of wildcard and hashed password; wildcard field will not be changed
/*
 * usage:
 
	<html>
	<head>
		<title>Login</title>
		<script src="Auth.class.js" language="JavaScript"></script>
	</head>
	<body>
		<form name="loginform" method="post">
			<input type="hidden" name="wildcard" value="@(WILDCARD)" />
			<input type="hidden" name="hash" />
			username: <input type="text" name="username" value="boomit" /><br />
			password: <input type="password" name="password" value="cretcher" /><br />
			<input type="submit" onclick="return LoginSHA1(document.loginform)" />
		</form>
	</body>
	</html>
 */

function LoginMD5(loginform) {
	loginform.hash.value = (new AuthMD5()).Scramble(loginform.password, loginform.wildcard);
	return true;
}

function LoginSHA1(loginform) {
	loginform.hash.value = (new AuthSHA1()).Scramble(loginform.password, loginform.wildcard);
	return true;
}

// Authentication Classes

function AuthMD5() {
/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
	this.hexcase = 1;  /* hex output format. 0 - lowercase; 1 - uppercase        */
	this.b64pad  = ""; /* base-64 pad character. "=" for strict RFC compliance   */
	this.chrsz   = 8;  /* bits per input character. 8 - ASCII; 16 - Unicode      */
}

/*
 * ================
 * Public interface
 * ================
 */
 
AuthMD5.prototype.Scramble = function(passwordField, wildcardField) {
	var pw = passwordField.value;
	passwordField.value = "";
	return this.hex_md5(wildcardField.value + pw).toUpperCase();
}
 
AuthMD5.prototype.Hash = function(source) {
	return this.hex_md5(source).toUpperCase();
}

/*
 * =====================
 * Semi-public interface
 * =====================
 */
 
	/*
	 * These are the functions you'll usually want to call
	 * They take string arguments and return either hex or base-64 encoded strings
	 */
AuthMD5.prototype.hex_md5 = function(s) { return this.binl2hex(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
AuthMD5.prototype.b64_md5 = function(s) { return this.binl2b64(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
AuthMD5.prototype.str_md5 = function(s) { return this.binl2str(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
AuthMD5.prototype.hex_hmac_md5 = function(key, data) { return this.binl2hex(this.core_hmac_md5(key, data)); }
AuthMD5.prototype.b64_hmac_md5 = function(key, data) { return this.binl2b64(this.core_hmac_md5(key, data)); }
AuthMD5.prototype.str_hmac_md5 = function(key, data) { return this.binl2str(this.core_hmac_md5(key, data)); }

	/*
	 * Perform a simple self-test to see if the VM is working
	 */
AuthMD5.prototype.md5_vm_test = function() {
  return this.hex_md5("abc").toLowerCase() == "900150983cd24fb0d6963f7d28e17f72";
}

/*
 * ========
 * Privates
 * ========
 */

	/*
	 * Calculate the MD5 of an array of little-endian words, and a bit length
	 */
AuthMD5.prototype.core_md5 = function(x, len) {
  /* append padding */
  x[len >> 5] |= 0x80 << ((len) % 32);
  x[(((len + 64) >>> 9) << 4) + 14] = len;

  var a =  1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d =  271733878;

  for(var i = 0; i < x.length; i += 16) {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;

    a = this.md5_ff(a, b, c, d, x[i+ 0], 7 , -680876936);
    d = this.md5_ff(d, a, b, c, x[i+ 1], 12, -389564586);
    c = this.md5_ff(c, d, a, b, x[i+ 2], 17,  606105819);
    b = this.md5_ff(b, c, d, a, x[i+ 3], 22, -1044525330);
    a = this.md5_ff(a, b, c, d, x[i+ 4], 7 , -176418897);
    d = this.md5_ff(d, a, b, c, x[i+ 5], 12,  1200080426);
    c = this.md5_ff(c, d, a, b, x[i+ 6], 17, -1473231341);
    b = this.md5_ff(b, c, d, a, x[i+ 7], 22, -45705983);
    a = this.md5_ff(a, b, c, d, x[i+ 8], 7 ,  1770035416);
    d = this.md5_ff(d, a, b, c, x[i+ 9], 12, -1958414417);
    c = this.md5_ff(c, d, a, b, x[i+10], 17, -42063);
    b = this.md5_ff(b, c, d, a, x[i+11], 22, -1990404162);
    a = this.md5_ff(a, b, c, d, x[i+12], 7 ,  1804603682);
    d = this.md5_ff(d, a, b, c, x[i+13], 12, -40341101);
    c = this.md5_ff(c, d, a, b, x[i+14], 17, -1502002290);
    b = this.md5_ff(b, c, d, a, x[i+15], 22,  1236535329);

    a = this.md5_gg(a, b, c, d, x[i+ 1], 5 , -165796510);
    d = this.md5_gg(d, a, b, c, x[i+ 6], 9 , -1069501632);
    c = this.md5_gg(c, d, a, b, x[i+11], 14,  643717713);
    b = this.md5_gg(b, c, d, a, x[i+ 0], 20, -373897302);
    a = this.md5_gg(a, b, c, d, x[i+ 5], 5 , -701558691);
    d = this.md5_gg(d, a, b, c, x[i+10], 9 ,  38016083);
    c = this.md5_gg(c, d, a, b, x[i+15], 14, -660478335);
    b = this.md5_gg(b, c, d, a, x[i+ 4], 20, -405537848);
    a = this.md5_gg(a, b, c, d, x[i+ 9], 5 ,  568446438);
    d = this.md5_gg(d, a, b, c, x[i+14], 9 , -1019803690);
    c = this.md5_gg(c, d, a, b, x[i+ 3], 14, -187363961);
    b = this.md5_gg(b, c, d, a, x[i+ 8], 20,  1163531501);
    a = this.md5_gg(a, b, c, d, x[i+13], 5 , -1444681467);
    d = this.md5_gg(d, a, b, c, x[i+ 2], 9 , -51403784);
    c = this.md5_gg(c, d, a, b, x[i+ 7], 14,  1735328473);
    b = this.md5_gg(b, c, d, a, x[i+12], 20, -1926607734);

    a = this.md5_hh(a, b, c, d, x[i+ 5], 4 , -378558);
    d = this.md5_hh(d, a, b, c, x[i+ 8], 11, -2022574463);
    c = this.md5_hh(c, d, a, b, x[i+11], 16,  1839030562);
    b = this.md5_hh(b, c, d, a, x[i+14], 23, -35309556);
    a = this.md5_hh(a, b, c, d, x[i+ 1], 4 , -1530992060);
    d = this.md5_hh(d, a, b, c, x[i+ 4], 11,  1272893353);
    c = this.md5_hh(c, d, a, b, x[i+ 7], 16, -155497632);
    b = this.md5_hh(b, c, d, a, x[i+10], 23, -1094730640);
    a = this.md5_hh(a, b, c, d, x[i+13], 4 ,  681279174);
    d = this.md5_hh(d, a, b, c, x[i+ 0], 11, -358537222);
    c = this.md5_hh(c, d, a, b, x[i+ 3], 16, -722521979);
    b = this.md5_hh(b, c, d, a, x[i+ 6], 23,  76029189);
    a = this.md5_hh(a, b, c, d, x[i+ 9], 4 , -640364487);
    d = this.md5_hh(d, a, b, c, x[i+12], 11, -421815835);
    c = this.md5_hh(c, d, a, b, x[i+15], 16,  530742520);
    b = this.md5_hh(b, c, d, a, x[i+ 2], 23, -995338651);

    a = this.md5_ii(a, b, c, d, x[i+ 0], 6 , -198630844);
    d = this.md5_ii(d, a, b, c, x[i+ 7], 10,  1126891415);
    c = this.md5_ii(c, d, a, b, x[i+14], 15, -1416354905);
    b = this.md5_ii(b, c, d, a, x[i+ 5], 21, -57434055);
    a = this.md5_ii(a, b, c, d, x[i+12], 6 ,  1700485571);
    d = this.md5_ii(d, a, b, c, x[i+ 3], 10, -1894986606);
    c = this.md5_ii(c, d, a, b, x[i+10], 15, -1051523);
    b = this.md5_ii(b, c, d, a, x[i+ 1], 21, -2054922799);
    a = this.md5_ii(a, b, c, d, x[i+ 8], 6 ,  1873313359);
    d = this.md5_ii(d, a, b, c, x[i+15], 10, -30611744);
    c = this.md5_ii(c, d, a, b, x[i+ 6], 15, -1560198380);
    b = this.md5_ii(b, c, d, a, x[i+13], 21,  1309151649);
    a = this.md5_ii(a, b, c, d, x[i+ 4], 6 , -145523070);
    d = this.md5_ii(d, a, b, c, x[i+11], 10, -1120210379);
    c = this.md5_ii(c, d, a, b, x[i+ 2], 15,  718787259);
    b = this.md5_ii(b, c, d, a, x[i+ 9], 21, -343485551);

    a = this.safe_add(a, olda);
    b = this.safe_add(b, oldb);
    c = this.safe_add(c, oldc);
    d = this.safe_add(d, oldd);
  }
  return Array(a, b, c, d);

}

	/*
	 * These functions implement the four basic operations the algorithm uses.
	 */
AuthMD5.prototype.md5_cmn = function(q, a, b, x, s, t) {
  return this.safe_add(this.bit_rol(this.safe_add(this.safe_add(a, q), this.safe_add(x, t)), s),b);
}
AuthMD5.prototype.md5_ff = function(a, b, c, d, x, s, t) {
  return this.md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
}
AuthMD5.prototype.md5_gg = function(a, b, c, d, x, s, t) {
  return this.md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
}
AuthMD5.prototype.md5_hh = function(a, b, c, d, x, s, t) {
  return this.md5_cmn(b ^ c ^ d, a, b, x, s, t);
}
AuthMD5.prototype.md5_ii = function(a, b, c, d, x, s, t) {
  return this.md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
}

	/*
	 * Calculate the HMAC-MD5, of a key and some data
	 */
AuthMD5.prototype.core_hmac_md5 = function(key, data) {
  var bkey = this.str2binl(key);
  if(bkey.length > 16) bkey = this.core_md5(bkey, key.length * this.chrsz);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++) {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = this.core_md5(ipad.concat(this.str2binl(data)), 512 + data.length * this.chrsz);
  return this.core_md5(opad.concat(hash), 512 + 128);
}

	/*
	 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
	 * to work around bugs in some JS interpreters.
	 */
AuthMD5.prototype.safe_add = function(x, y) {
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
}

	/*
	 * Bitwise rotate a 32-bit number to the left.
	 */
AuthMD5.prototype.bit_rol = function(num, cnt) {
  return (num << cnt) | (num >>> (32 - cnt));
}

	/*
	 * Convert a string to an array of little-endian words
	 * If chrsz is ASCII, characters >255 have their hi-byte silently ignored.
	 */
AuthMD5.prototype.str2binl = function(str) {
  var bin = Array();
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < str.length * this.chrsz; i += this.chrsz)
    bin[i>>5] |= (str.charCodeAt(i / this.chrsz) & mask) << (i%32);
  return bin;
}

	/*
	 * Convert an array of little-endian words to a string
	 */
AuthMD5.prototype.binl2str = function(bin) {
  var str = "";
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < bin.length * 32; i += this.chrsz)
    str += String.fromCharCode((bin[i>>5] >>> (i % 32)) & mask);
  return str;
}

	/*
	 * Convert an array of little-endian words to a hex string.
	 */
AuthMD5.prototype.binl2hex = function(binarray) {
  var hex_tab = this.hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i++) {
    str += hex_tab.charAt((binarray[i>>2] >> ((i%4)*8+4)) & 0xF) +
           hex_tab.charAt((binarray[i>>2] >> ((i%4)*8  )) & 0xF);
  }
  return str;
}

	/*
	 * Convert an array of little-endian words to a base-64 string
	 */
AuthMD5.prototype.binl2b64 = function(binarray) {
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i += 3) {
    var triplet = (((binarray[i   >> 2] >> 8 * ( i   %4)) & 0xFF) << 16)
                | (((binarray[i+1 >> 2] >> 8 * ((i+1)%4)) & 0xFF) << 8 )
                |  ((binarray[i+2 >> 2] >> 8 * ((i+2)%4)) & 0xFF);
    for(var j = 0; j < 4; j++) {
      if(i * 8 + j * 6 > binarray.length * 32) str += this.b64pad;
      else str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
    }
  }
  return str;
}


/*
 * A JavaScript implementation of the Secure Hash Algorithm, SHA-1, as defined
 * in FIPS PUB 180-1
 * Version 2.1 Copyright Paul Johnston 2000 - 2002.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for details.
 *
 * Class wrapper by Matt Newberry - 2004-03-26
 */

 function AuthSHA1() {
/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
	this.hexcase = 1;  /* hex output format. 0 - lowercase; 1 - uppercase        */
	this.b64pad  = ""; /* base-64 pad character. "=" for strict RFC compliance   */
	this.chrsz   = 8;  /* bits per input character. 8 - ASCII; 16 - Unicode      */
}

/*
 * ================
 * Public interface
 * ================
 */

AuthSHA1.prototype.Scramble = function(passwordField, wildcardField) {
	var pw = this.hex_sha1(passwordField.value).toUpperCase();	// server-side passwords are already hashed
	passwordField.value = "";
	return this.hex_sha1(wildcardField.value + pw).toUpperCase();
}
 
AuthSHA1.prototype.Hash = function(source) {
	return this.hex_sha1(source).toUpperCase();
}

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */
AuthSHA1.prototype.hex_sha1 = function(s) { return this.binb2hex(this.core_sha1(this.str2binb(s), s.length * this.chrsz)); }
AuthSHA1.prototype.b64_sha1 = function(s) { return this.binb2b64(this.core_sha1(this.str2binb(s), s.length * this.chrsz)); }
AuthSHA1.prototype.str_sha1 = function(s) { return this.binb2str(this.core_sha1(this.str2binb(s), s.length * this.chrsz)); }
AuthSHA1.prototype.hex_hmac_sha1 = function(key, data) { return this.binb2hex(this.core_hmac_sha1(key, data)); }
AuthSHA1.prototype.b64_hmac_sha1 = function(key, data) { return this.binb2b64(this.core_hmac_sha1(key, data)); }
AuthSHA1.prototype.str_hmac_sha1 = function(key, data) { return this.binb2str(this.core_hmac_sha1(key, data)); }

/*
 * Perform a simple self-test to see if the VM is working
 */
AuthSHA1.prototype.sha1_vm_test = function() {
  return this.hex_sha1("abc").toLowerCase() == "a9993e364706816aba3e25717850c26c9cd0d89d";
}

/*
 * Calculate the SHA-1 of an array of big-endian words, and a bit length
 */
AuthSHA1.prototype.core_sha1 = function(x, len) {
		/* append padding */
	x[len >> 5] |= 0x80 << (24 - len % 32);
	x[((len + 64 >> 9) << 4) + 15] = len;
	
	var w = Array(80);
	var a =  1732584193;
	var b = -271733879;
	var c = -1732584194;
	var d =  271733878;
	var e = -1009589776;
	
  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;
    var olde = e;

    for(var j = 0; j < 80; j++)
    {
      if(j < 16) w[j] = x[i + j];
      else w[j] = this.rol(w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16], 1);
      var t = this.safe_add(this.safe_add(this.rol(a, 5), this.sha1_ft(j, b, c, d)), 
                       this.safe_add(this.safe_add(e, w[j]), this.sha1_kt(j)));
      e = d;
      d = c;
      c = this.rol(b, 30);
      b = a;
      a = t;
    }

    a = this.safe_add(a, olda);
    b = this.safe_add(b, oldb);
    c = this.safe_add(c, oldc);
    d = this.safe_add(d, oldd);
    e = this.safe_add(e, olde);
  }
  return Array(a, b, c, d, e);
}

/*
 * Perform the appropriate triplet combination function for the current
 * iteration
 */
AuthSHA1.prototype.sha1_ft = function(t, b, c, d) {
  if(t < 20) return (b & c) | ((~b) & d);
  if(t < 40) return b ^ c ^ d;
  if(t < 60) return (b & c) | (b & d) | (c & d);
  return b ^ c ^ d;
}

/*
 * Determine the appropriate additive constant for the current iteration
 */
AuthSHA1.prototype.sha1_kt = function(t) {
  return (t < 20) ?  1518500249 : (t < 40) ?  1859775393 :
         (t < 60) ? -1894007588 : -899497514;
}  

/*
 * Calculate the HMAC-SHA1 of a key and some data
 */
AuthSHA1.prototype.core_hmac_sha1 = function(key, data) {
  var bkey = this.str2binb(key);
  if(bkey.length > 16) bkey = this.core_sha1(bkey, key.length * this.chrsz);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++) 
  {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = this.core_sha1(ipad.concat(this.str2binb(data)), 512 + data.length * this.chrsz);
  return this.core_sha1(opad.concat(hash), 512 + 160);
}

/*
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 */
AuthSHA1.prototype.safe_add = function(x, y) {
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
}

/*
 * Bitwise rotate a 32-bit number to the left.
 */
AuthSHA1.prototype.rol = function(num, cnt) {
  return (num << cnt) | (num >>> (32 - cnt));
}

/*
 * Convert an 8-bit or 16-bit string to an array of big-endian words
 * In 8-bit function, characters >255 have their hi-byte silently ignored.
 */
AuthSHA1.prototype.str2binb = function(str) {
  var bin = Array();
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < str.length * this.chrsz; i += this.chrsz)
    bin[i>>5] |= (str.charCodeAt(i / this.chrsz) & mask) << (24 - i%32);
  return bin;
}

/*
 * Convert an array of big-endian words to a string
 */
AuthSHA1.prototype.binb2str = function(bin) {
  var str = "";
  var mask = (1 << this.chrsz) - 1;
  for(var i = 0; i < bin.length * 32; i += this.chrsz)
    str += String.fromCharCode((bin[i>>5] >>> (24 - i%32)) & mask);
  return str;
}

/*
 * Convert an array of big-endian words to a hex string.
 */
AuthSHA1.prototype.binb2hex = function(binarray) {
  var hex_tab = this.hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i++)
  {
    str += hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8+4)) & 0xF) +
           hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8  )) & 0xF);
  }
  return str;
}

/*
 * Convert an array of big-endian words to a base-64 string
 */
AuthSHA1.prototype.binb2b64 = function(binarray) {
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var str = "";
  for(var i = 0; i < binarray.length * 4; i += 3)
  {
    var triplet = (((binarray[i   >> 2] >> 8 * (3 -  i   %4)) & 0xFF) << 16)
                | (((binarray[i+1 >> 2] >> 8 * (3 - (i+1)%4)) & 0xFF) << 8 )
                |  ((binarray[i+2 >> 2] >> 8 * (3 - (i+2)%4)) & 0xFF);
    for(var j = 0; j < 4; j++)
    {
      if(i * 8 + j * 6 > binarray.length * 32) str += this.b64pad;
      else str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
    }
  }
  return str;
}
